"""
consent.py - Consent management logic
"""

# In-memory consent store (for demonstration purposes)
consent_store = {}

def register_consent(user_id, consent_data):
    """
    Register user consent.
    :param user_id: str
    :param consent_data: dict
    """
    consent_store[user_id] = {**consent_data, 'revoked': False}

def update_consent(user_id, consent_data):
    """
    Update user consent.
    :param user_id: str
    :param consent_data: dict
    """
    if user_id in consent_store:
        consent_store[user_id].update(consent_data)

def revoke_consent(user_id):
    """
    Revoke user consent.
    :param user_id: str
    """
    if user_id in consent_store:
        consent_store[user_id]['revoked'] = True

def is_consent_active(user_id):
    """
    Check if user consent is active.
    :param user_id: str
    :return: bool
    """
    return user_id in consent_store and not consent_store[user_id]['revoked']


"""
retention.py - Retention policy logic
"""
from datetime import datetime

# In-memory retention store (for demonstration purposes)
retention_store = {}

def mark_for_expiration(user_id, expiration_date):
    """
    Mark data for expiration/removal.
    :param user_id: str
    :param expiration_date: datetime
    """
    retention_store[user_id] = {'expiration_date': expiration_date}

def is_expired(user_id):
    """
    Check if data is expired.
    :param user_id: str
    :return: bool
    """
    if user_id not in retention_store:
        return False
    return datetime.now() > retention_store[user_id]['expiration_date']


"""
masking.py - Masking logic for personal data
"""
import re

def mask(data_type, value):
    """
    Mask a sensitive value (e.g., CPF, email, phone, RG, CNPJ, name, address) according to type.
    :param data_type: str - The type of data to mask (e.g., 'cpf', 'email', 'phone', 'rg', 'cnpj', 'name', 'address').
    :param value: str - The value to be masked.
    :return: str - The masked value.
    """
    if data_type == 'cpf':
        # Mask all but the last 2 digits
        return re.sub(r'\d(?=\d{2})', '*', value)
    if data_type == 'email':
        # Mask all but the first and last character before @
        user, domain = value.split('@')
        if len(user) <= 2:
            return '*@' + domain
        return user[0] + '*' * (len(user) - 2) + user[-1] + '@' + domain
    if data_type == 'phone':
        # Mask all but the last 2 digits
        return re.sub(r'\d(?=\d{2})', '*', value)
    if data_type == 'rg':
        # Mask all but the last 2 digits
        return re.sub(r'\d(?=\d{2})', '*', value)
    if data_type == 'cnpj':
        # Mask all but the last 4 digits
        return re.sub(r'\d(?=\d{4})', '*', value)
    if data_type == 'name':
        # Mask all but the first and last character of each word
        return ' '.join([
            w[0] + '*' * (len(w) - 2) + w[-1] if len(w) > 2 else w[0] + '*' for w in value.split()
        ])
    if data_type == 'address':
        # Mask all but the first and last character of each word, except for numbers
        def mask_word(w):
            if w.isdigit():
                return w
            if len(w) <= 2:
                return w[0] + '*'
            return w[0] + '*' * (len(w) - 2) + w[-1]
        return ' '.join([mask_word(w) for w in value.split()])
    return value


"""
PrivacySchema main module
"""

from .validation import validate, is_valid_cpf, is_valid_email
from .masking import mask
from .consent import register_consent, update_consent, revoke_consent, is_consent_active
from .retention import mark_for_expiration, is_expired
from .audit import log_operation, get_audit_logs

# Expose the main API
__all__ = [
    'validate', 'is_valid_cpf', 'is_valid_email',
    'mask',
    'register_consent', 'update_consent', 'revoke_consent', 'is_consent_active',
    'mark_for_expiration', 'is_expired',
    'log_operation', 'get_audit_logs'
]


"""
validation.py - Validation logic for personal data
"""
import re

def validate(data):
    """
    Validate if the given data contains valid personal information.
    Currently supports CPF and email validation.
    :param data: dict - The user data to validate.
    :return: dict - Validation results for each field.
    """
    result = {}
    if 'cpf' in data:
        result['cpf'] = is_valid_cpf(data['cpf'])
    if 'email' in data:
        result['email'] = is_valid_email(data['email'])
    # Add more validations as needed
    return result

def is_valid_cpf(cpf):
    """
    Validate CPF (Brazilian ID).
    :param cpf: str
    :return: bool
    """
    cpf = re.sub(r'\D', '', cpf)
    if len(cpf) != 11 or cpf == cpf[0] * 11:
        return False
    sum1 = sum(int(cpf[i]) * (10 - i) for i in range(9))
    d1 = ((sum1 * 10) % 11) % 10
    sum2 = sum(int(cpf[i]) * (11 - i) for i in range(10))
    d2 = ((sum2 * 10) % 11) % 10
    return d1 == int(cpf[9]) and d2 == int(cpf[10])

def is_valid_email(email):
    """
    Validate email address.
    :param email: str
    :return: bool
    """
    return re.match(r'^[^@\s]+@[^@\s]+\.[^@\s]+$', email) is not None


import os
from setuptools import setup, find_packages

# Corrigindo o caminho do README.md para o diretório correto
this_directory = os.path.abspath(os.path.dirname(__file__))
readme_path = os.path.join(this_directory, '../README.md')
if not os.path.exists(readme_path):
    # Tenta buscar um nível acima, caso não exista no diretório atual
    readme_path = os.path.join(this_directory, 'README.md')

with open(readme_path, encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="privacyschema",
    version="0.3.0",
    description="Pluggable framework for personal data compliance (LGPD/GDPR)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Julio Amorim",
    author_email="julio@grupojpc.com.br",
    url="https://github.com/julioamorimdev/PrivacySchema",
    packages=find_packages(),
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Security",
        "Topic :: Software Development :: Libraries",
    ],
    python_requires=">=3.6",
    install_requires=[],
    include_package_data=True,
)


"""
audit.py - Audit logging for sensitive data operations
"""
from datetime import datetime

# In-memory audit log (for demonstration purposes)
audit_log = []

def log_operation(user_id, operation, details):
    """
    Log an operation on sensitive data.
    :param user_id: str
    :param operation: str
    :param details: dict
    """
    audit_log.append({
        'timestamp': datetime.now().isoformat(),
        'user_id': user_id,
        'operation': operation,
        'details': details
    })

def get_audit_logs():
    """
    Get all audit logs.
    :return: list
    """
    return audit_log